@@ -60,19 +60,23 @@
 #'   It is now recommended to specify those matrices in the formula
 #'   interface using the \code{\link{gr}} and related functions. See
 #'   \code{vignette("brms_phylogenetics")} for more details.
-#' @param save_ranef A flag to indicate if group-level effects for each level of
-#'   the grouping factor(s) should be saved (default is \code{TRUE}). Set to
-#'   \code{FALSE} to save memory. The argument has no impact on the model
-#'   fitting itself.
-#' @param save_mevars A flag to indicate if samples of latent noise-free
-#'   variables obtained by using \code{me} and \code{mi} terms should be saved
-#'   (default is \code{FALSE}). Saving these samples allows to better use
-#'   methods such as \code{predict} with the latent variables but leads to very
-#'   large \R objects even for models of moderate size and complexity.
-#' @param save_all_pars A flag to indicate if samples from all variables defined
-#'   in Stan's \code{parameters} block should be saved (default is
-#'   \code{FALSE}). Saving these samples is required in order to apply the
-#'   methods \code{bridge_sampler}, \code{bayes_factor}, and \code{post_prob}.
+#' @param save_pars An object generated by \code{\link{save_pars}} controlling
+#'   which parameters should be saved in the model. The argument has no
+#'   impact on the model fitting itself.
+#' @param save_ranef (Deprecated) A flag to indicate if group-level effects for
+#'   each level of the grouping factor(s) should be saved (default is
+#'   \code{TRUE}). Set to \code{FALSE} to save memory. The argument has no
+#'   impact on the model fitting itself.
+#' @param save_mevars (Deprecated) A flag to indicate if samples of latent
+#'   noise-free variables obtained by using \code{me} and \code{mi} terms should
+#'   be saved (default is \code{FALSE}). Saving these samples allows to better
+#'   use methods such as \code{predict} with the latent variables but leads to
+#'   very large \R objects even for models of moderate size and complexity.
+#' @param save_all_pars (Deprecated) A flag to indicate if samples from all
+#'   variables defined in Stan's \code{parameters} block should be saved
+#'   (default is \code{FALSE}). Saving these samples is required in order to
+#'   apply the methods \code{bridge_sampler}, \code{bayes_factor}, and
+#'   \code{post_prob}.
 #' @param sample_prior Indicate if samples from priors should be drawn
 #'   additionally to the posterior samples. Options are \code{"no"} (the
 #'   default), \code{"yes"}, and \code{"only"}. Among others, these samples can
@@ -131,18 +135,25 @@
 #'   be as many processors as the hardware and RAM allow (up to the number of
 #'   chains). For non-Windows OS in non-interactive \R sessions, forking is used
 #'   instead of PSOCK clusters.
+#' @param threads Number of threads to use in within-chain parallelization. For
+#'   more control over the threading process, \code{threads} may also be a
+#'   \code{brmsthreads} object created by \code{\link{threading}}. Within-chain
+#'   parallelization is experimental! We recommend its use only if you are
+#'   experienced with Stan's \code{reduce_sum} function and have a slow running
+#'   model that cannot be sped up by any other means.
 #' @param algorithm Character string naming the estimation approach to use.
 #'   Options are \code{"sampling"} for MCMC (the default), \code{"meanfield"} for
-#'   variational inference with independent normal distributions, or
+#'   variational inference with independent normal distributions,
 #'   \code{"fullrank"} for variational inference with a multivariate normal
-#'   distribution. Can be set globally for the current \R session via the
-#'   \code{"stan_algorithm"} option (see \code{\link{options}}).
+#'   distribution, or \code{"fixed_param"} for sampling from fixed parameter
+#'   values. Can be set globally for the current \R session via the
+#'   \code{"brms.algorithm"} option (see \code{\link{options}}).
 #' @param backend Character string naming the package to use as the backend for
 #'   fitting the Stan model. Options are \code{"rstan"} (the default) or
 #'   \code{"cmdstanr"}. Can be set globally for the current \R session via the
-#'   \code{"stan_backend"} option (see \code{\link{options}}). Details on the
+#'   \code{"brms.backend"} option (see \code{\link{options}}). Details on the
 #'   \pkg{rstan} and \pkg{cmdstanr} packages are available at
-#'   \url{https://mc-stan.org/rstan} and \url{https://mc-stan.org/cmdstanr},
+#'   \url{https://mc-stan.org/rstan/} and \url{https://mc-stan.org/cmdstanr/},
 #'   respectively.
 #' @param control A named \code{list} of parameters to control the sampler's
 #'   behavior. It defaults to \code{NULL} so all the default values are used.
@@ -262,12 +273,12 @@
 #' @examples
 #' \dontrun{
 #' # Poisson regression for the number of seizures in epileptic patients
-#' # using student_t priors for population-level effects
-#' # and half cauchy priors for standard deviations of group-level effects
-#' bprior1 <- prior(student_t(5,0,10), class = b) +
+#' # using normal priors for population-level effects
+#' # and half-cauchy priors for standard deviations of group-level effects
+#' prior1 <- prior(normal(0,10), class = b) +
 #'   prior(cauchy(0,2), class = sd)
 #' fit1 <- brm(count ~ zAge + zBase * Trt + (1|patient),
-#'             data = epilepsy, family = poisson(), prior = bprior1)
+#'             data = epilepsy, family = poisson(), prior = prior1)
 #'
 #' # generate a summary of the results
 #' summary(fit1)
@@ -315,16 +326,21 @@
 #' summary(fit4)
 #'
 #'
-#' # Simple non-linear gaussian model
-#' x <- rnorm(100)
-#' y <- rnorm(100, mean = 2 - 1.5^x, sd = 1)
-#' data5 <- data.frame(x, y)
-#' bprior5 <- prior(normal(0, 2), nlpar = a1) +
-#'   prior(normal(0, 2), nlpar = a2)
-#' fit5 <- brm(bf(y ~ a1 - a2^x, a1 + a2 ~ 1, nl = TRUE),
-#'             data = data5, prior = bprior5)
+#' # Non-linear Gaussian model
+#' fit5 <- brm(
+#'   bf(cum ~ ult * (1 - exp(-(dev/theta)^omega)),
+#'      ult ~ 1 + (1|AY), omega ~ 1, theta ~ 1, 
+#'      nl = TRUE),
+#'   data = loss, family = gaussian(),
+#'   prior = c(
+#'     prior(normal(5000, 1000), nlpar = "ult"),
+#'     prior(normal(1, 2), nlpar = "omega"),
+#'     prior(normal(45, 10), nlpar = "theta")
+#'   ),
+#'   control = list(adapt_delta = 0.9)
+#' )
 #' summary(fit5)
-#' plot(conditional_effects(fit5), ask = FALSE)
+#' conditional_effects(fit5)
 #'
 #'
 #' # Normal model with heterogeneous variances
@@ -375,13 +391,15 @@
 brm <- function(formula, data, family = gaussian(), prior = NULL, 
                 autocor = NULL, data2 = NULL, cov_ranef = NULL, 
                 sample_prior = "no", sparse = NULL, knots = NULL,
-                stanvars = NULL, stan_funs = NULL, fit = NA, save_ranef = TRUE, 
-                save_mevars = FALSE, save_all_pars = FALSE, 
+                stanvars = NULL, stan_funs = NULL, fit = NA, 
+                save_pars = NULL, save_ranef = NULL, 
+                save_mevars = NULL, save_all_pars = NULL, 
                 inits = "random", chains = 4, iter = 2000, 
                 warmup = floor(iter / 2), thin = 1,
-                cores = getOption("mc.cores", 1L), control = NULL,
-                algorithm = getOption("stan_algorithm", "sampling"),
-                backend = getOption("stan_backend", "rstan"),
+                cores = getOption("mc.cores", 1), 
+                threads = NULL, control = NULL,
+                algorithm = getOption("brms.algorithm", "sampling"),
+                backend = getOption("brms.backend", "rstan"),
                 future = getOption("future", FALSE), silent = TRUE, 
                 seed = NA, save_model = NULL, stan_model_args = list(),
                 file = NULL, empty = FALSE, rename = TRUE, ...) {
@@ -395,7 +413,6 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
   }
   
   # validate arguments later passed to Stan
-  dots <- list(...)
   algorithm <- match.arg(algorithm, algorithm_choices())
   backend <- match.arg(backend, backend_choices())
   silent <- as_one_logical(silent)
@@ -404,6 +421,7 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
   thin <- as_one_numeric(thin)
   chains <- as_one_numeric(chains)
   cores <- as_one_numeric(cores)
+  threads <- validate_threads(threads)
   future <- as_one_logical(future) && chains > 0L
   seed <- as_one_numeric(seed, allow_na = TRUE)
   empty <- as_one_logical(empty)
@@ -414,8 +432,10 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
     # re-use existing model
     x <- fit
     x$criteria <- list()
+    backend <- x$backend
     sdata <- standata(x)
     model <- compiled_model(x)
+    exclude <- exclude_pars(x)
   } else {  
     # build new model
     formula <- validate_formula(
@@ -438,27 +458,31 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
       sample_prior = sample_prior
     )
     stanvars <- validate_stanvars(stanvars, stan_funs = stan_funs)
-    # initialize S3 object
-    x <- brmsfit(
-      formula = formula, data = data, data2 = data2, prior = prior, 
-      stanvars = stanvars, algorithm = algorithm, backend = backend,
-      family = family
-    )
-    x$ranef <- tidy_ranef(bterms, data = x$data)  
-    x$exclude <- exclude_pars(
-      x, save_ranef = save_ranef, 
+    save_pars <- validate_save_pars(
+      save_pars, save_ranef = save_ranef, 
       save_mevars = save_mevars,
       save_all_pars = save_all_pars
     )
-    x$model <- .make_stancode(
+    ranef <- tidy_ranef(bterms, data = data)
+    # generate Stan code
+    model <- .make_stancode(
       bterms, data = data, prior = prior, 
-      stanvars = stanvars, save_model = save_model
+      stanvars = stanvars, save_model = save_model,
+      backend = backend, threads = threads
     )
+    # initialize S3 object
+    x <- brmsfit(
+      formula = formula, data = data, data2 = data2, prior = prior, 
+      stanvars = stanvars, model = model, algorithm = algorithm, 
+      backend = backend, threads = threads, save_pars = save_pars, 
+      ranef = ranef, family = family
+    )
+    exclude <- exclude_pars(x)
     # generate Stan data before compiling the model to avoid
     # unnecessary compilations in case of invalid data
     sdata <- .make_standata(
-      bterms, data = data, prior = prior, 
-      data2 = data2, stanvars = stanvars
+      bterms, data = data, prior = prior, data2 = data2, 
+      stanvars = stanvars, threads = threads
     )
     if (empty) {
       # return the brmsfit object with an empty 'fit' slot
@@ -466,16 +490,16 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
     }
     # compile the Stan model
     compile_args <- stan_model_args
-    compile_args$model <- x$model
+    compile_args$model <- model
     compile_args$backend <- backend
+    compile_args$threads <- threads
     model <- do_call(compile_model, compile_args)
   }
   
   # fit the Stan model
   fit_args <- nlist(
     model, sdata, algorithm, backend, iter, warmup, thin, chains, 
-    cores, inits, exclude = x$exclude, control, future, seed, 
-    silent, ...
+    cores, threads, inits, exclude, control, future, seed, silent, ...
   )
   x$fit <- do_call(fit_model, fit_args)
 
