@@ -66,8 +66,9 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
       )
     }
   }
-  # ensure compatibility with objects not created in the current R session
-  x$fit@.MISC <- suppressMessages(brm(fit = x, chains = 0))$fit@.MISC
+  require_backend("rstan", x)
+  # otherwise loo_moment_match might not work in a new R session
+  x <- update_misc_env(x)
   out <- try(loo::loo_moment_match.default(
     x, loo = loo, 
     post_draws = as.matrix, 
@@ -93,20 +94,23 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   as.vector(log_lik(x, newdata = newdata[i, , drop = FALSE], ...))
 }
 
-# transform parameters to the unconstraint space
+# transform parameters to the unconstrained space
 .unconstrain_pars <- function(x, pars, ...) {
   unconstrain_pars_stanfit(x$fit, pars = pars, ...)
 }
 
 # compute log_prob for each posterior draws on the unconstrained space
 .log_prob_upars <- function(x, upars, ...) {
+  if (os_is_windows()) {
+    x <- update_misc_env(x) 
+  }
   log_prob_upars_stanfit(x$fit, upars = upars, ...)
 }
 
 # transform parameters to the constraint space
 .update_pars <- function(x, upars, ...) {
   # list with one element per posterior draw
-  pars <- apply(upars, 1, rstan::constrain_pars, object = x$fit)
+  pars <- apply(upars, 1, .constrain_pars, x = x)
   # transform samples
   nsamples <- length(pars)
   pars <- unlist(pars)
@@ -141,10 +145,21 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   rename_pars(x)
 }
 
+# wrapper around rstan::constrain_pars
+# ensures that the right posterior samples are excluded
+.constrain_pars <- function(upars, x) {
+  out <- rstan::constrain_pars(upars, object = x$fit)
+  out[x$exclude] <- NULL
+  out
+}
+
 # compute log_lik values based on the unconstrained parameters
 .log_lik_i_upars <- function(x, upars, i, samples = NULL, 
                              subset = NULL, ...) {
   # do not pass subset or nsamples further to avoid subsetting twice
+  if (os_is_windows()) {
+    x <- update_misc_env(x) 
+  }
   x <- .update_pars(x, upars = upars, ...)
   .log_lik_i(x, i = i, ...)
 }
