@@ -9,11 +9,16 @@ stan_response <- function(bterms, data) {
   multicol <- has_multicol(family)
   px <- check_prefix(bterms)
   resp <- usc(combine_prefix(px))
-  out <- list()
-  str_add(out$data) <- glue(
-    "  int<lower=1> N{resp};  // number of observations\n"
-  )
-  
+  out <- list(resp_type = rtype)
+  if (nzchar(resp)) {
+    # global N is defined elsewhere
+    str_add(out$data) <- glue(
+      "  int<lower=1> N{resp};  // number of observations\n"
+    )
+    str_add(out$pll_def) <- glue(
+      "  int N{resp} = end - start + 1;\n"
+    )
+  }
   if (has_cat(family)) {
     str_add(out$data) <- glue(
       "  int<lower=2> ncat{resp};  // number of categories\n"
@@ -24,21 +29,25 @@ stan_response <- function(bterms, data) {
       str_add(out$data) <- glue(
         "  vector[ncat{resp}] Y{resp}[N{resp}];  // response array\n"
       )
+      str_add(out$pll_args) <- glue(", vector[] Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
         "  int Y{resp}[N{resp}, ncat{resp}];  // response array\n"
       )
+      str_add(out$pll_args) <- glue(", int[,] Y{resp}")
     }
   } else {
     if (rtype == "real") {
-      # don't use real Y[n]
+      # type vector (instead of real[]) is required by some PDFs
       str_add(out$data) <- glue(
         "  vector[N{resp}] Y{resp};  // response variable\n"
       )
+      str_add(out$pll_args) <- glue(", vector Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
         "  int Y{resp}[N{resp}];  // response variable\n"
       )
+      str_add(out$pll_args) <- glue(", int[] Y{resp}")
     }
   }
   if (has_ndt(family)) {
@@ -50,11 +59,13 @@ stan_response <- function(bterms, data) {
     str_add(out$data) <- glue(
       "  int trials{resp}[N{resp}];  // number of trials\n"
     )
+    str_add(out$pll_args) <- glue(", int[] trials{resp}")
   }
   if (is.formula(bterms$adforms$weights)) {
     str_add(out$data) <- glue(
       "  vector<lower=0>[N{resp}] weights{resp};  // model weights\n" 
     )
+    str_add(out$pll_args) <- glue(", vector weights{resp}")
   }
   if (has_thres(family)) {
     groups <- get_thres_groups(family)
@@ -80,10 +91,12 @@ stan_response <- function(bterms, data) {
         "    Kthres_end{resp}[i] = Kthres_end{resp}[i-1] + nthres{resp}[i];\n",
         "  }}\n"
       )
+      str_add(out$pll_args) <- glue(", int nthres{resp}, int[,] Jthres{resp}")
     } else {
       str_add(out$data) <- glue(
         "  int<lower=2> nthres{resp};  // number of thresholds\n"
       )
+      str_add(out$pll_args) <- glue(", int nthres{resp}")
     }
   }
   if (is.formula(bterms$adforms$se)) {
@@ -93,11 +106,13 @@ stan_response <- function(bterms, data) {
     str_add(out$tdata_def) <- glue(
       "  vector<lower=0>[N{resp}] se2{resp} = square(se{resp});\n"
     )
+    str_add(out$pll_args) <- glue(", vector se{resp}, vector se2{resp}")
   }
   if (is.formula(bterms$adforms$dec)) {
     str_add(out$data) <- glue(
       "  int<lower=0,upper=1> dec{resp}[N{resp}];  // decisions\n"
     )
+    str_add(out$pll_args) <- glue(", int[] dec{resp}")
   }
   if (is.formula(bterms$adforms$rate)) {
     str_add(out$data) <- glue(
@@ -108,21 +123,30 @@ stan_response <- function(bterms, data) {
       "  // log response denominator\n",
       "  vector[N{resp}] log_denom{resp} = log(denom{resp});\n"
     )
+    str_add(out$pll_args) <- glue(
+      ", vector denom{resp}, vector log_denom{resp}"
+    )
   }
   if (is.formula(bterms$adforms$cens)) {
     cens <- eval_rhs(bterms$adforms$cens)
     str_add(out$data) <- glue(
       "  int<lower=-1,upper=2> cens{resp}[N{resp}];  // indicates censoring\n"
     )
+    str_add(out$pll_args) <- glue(", int[] cens{resp}")
     if (cens$vars$y2 != "NA") {
       # interval censoring is required
-      rcens <- str_if(rtype == "int", 
-                      glue("  int rcens{resp}[N{resp}];"), 
-                      glue("  vector[N{resp}] rcens{resp};")
-      )
-      str_add(out$data) <- glue(
-        rcens, "  // right censor points for interval censoring\n"
-      )
+      if (rtype == "int") {
+        str_add(out$data) <- glue(
+          "  int rcens{resp}[N{resp}];"
+        )
+        str_add(out$pll_args) <- glue(", int[] rcens{resp}")
+      } else {
+        str_add(out$data) <- glue(
+          "  vector[N{resp}] rcens{resp};"
+        )
+        str_add(out$pll_args) <- glue(", vector rcens{resp}")
+      }
+      str_add(out$data) <- "  // right censor points for interval censoring\n"
     }
   }
   bounds <- trunc_bounds(bterms, data = data)
@@ -130,11 +154,13 @@ stan_response <- function(bterms, data) {
     str_add(out$data) <- glue(
       "  {rtype} lb{resp}[N{resp}];  // lower truncation bounds;\n"
     )
+    str_add(out$pll_args) <- glue(", {rtype}[] lb{resp}")
   }
   if (any(bounds$ub < Inf)) {
     str_add(out$data) <- glue(
       "  {rtype} ub{resp}[N{resp}];  // upper truncation bounds\n"
     )
+    str_add(out$pll_args) <- glue(", {rtype}[] ub{resp}")
   }
   if (is.formula(bterms$adforms$mi)) {
     # TODO: pass 'Ybounds' via 'standata' instead of hardcoding them
@@ -149,13 +175,14 @@ stan_response <- function(bterms, data) {
       str_add(out$par) <- glue(
         "  vector{Ybounds}[Nmi{resp}] Ymi{resp};  // estimated missings\n"
       )
-      str_add(out$model_def) <- glue(
+      str_add(out$model_no_pll_def) <- glue(
         "  // vector combining observed and missing responses\n",
         "  vector[N{resp}] Yl{resp} = Y{resp};\n" 
       )
-      str_add(out$model_comp_basic) <- glue(
+      str_add(out$model_no_pll_comp_basic) <- glue(
         "  Yl{resp}[Jmi{resp}] = Ymi{resp};\n"
       )
+      str_add(out$pll_args) <- glue(", vector Yl{resp}")
     } else {
       str_add(out$data) <- glue(
         "  // data for measurement-error in the response\n",
@@ -169,9 +196,9 @@ stan_response <- function(bterms, data) {
       )
       str_add(out$prior) <- glue(
         "  target += normal_lpdf(Y{resp}[Jme{resp}]",
-        " | Yl{resp}[Jme{resp}],", 
-        " noise{resp}[Jme{resp}]);\n"
+        " | Yl{resp}[Jme{resp}], noise{resp}[Jme{resp}]);\n"
       )
+      str_add(out$pll_args) <- glue(", vector Yl{resp}")
     }
   }
   if (is.formula(bterms$adforms$vreal)) {
@@ -180,8 +207,9 @@ stan_response <- function(bterms, data) {
     k <- length(vreal$vars)
     str_add(out$data) <- cglue(
       "  // data for custom real vectors\n",
-      "  vector[N{resp}] vreal{seq_len(k)}{resp};\n"
+      "  real vreal{seq_len(k)}{resp}[N{resp}];\n"
     )
+    str_add(out$pll_args) <- cglue(", real[] vreal{seq_len(k)}{resp}")
   }
   if (is.formula(bterms$adforms$vint)) {
     # vectors of integer values for use in custom families
@@ -191,6 +219,7 @@ stan_response <- function(bterms, data) {
       "  // data for custom integer vectors\n",
       "  int vint{seq_len(k)}{resp}[N{resp}];\n"
     )
+    str_add(out$pll_args) <- cglue(", int[] vint{seq_len(k)}{resp}")
   }
   out
 }
@@ -280,21 +309,25 @@ stan_thres <- function(bterms, data, prior, ...) {
       "  vector[nthres{resp}{grb}] Intercept{p}_stz{gr};",
       "  // sum-to-zero constraint thresholds\n"
     )
+    str_add(out$tpar_comp) <- "  // compute sum-to-zero constraint thresholds\n"
     str_add(out$tpar_comp) <- cglue(
-      "  // compute sum-to-zero constraint thresholds\n",
       "  Intercept{p}_stz{gr} = Intercept{p}{gr} - mean(Intercept{p}{gr});\n"
-    ) 
+    )
   }
   if (has_thres_groups(bterms)) {
     # merge all group specific thresholds into one vector
-    str_add(out$model_def) <- glue(
+    str_add(out$tpar_def) <- glue(
       "  vector[nmthres{resp}] merged_Intercept{p}{stz};  // merged thresholds\n"
     )
+    str_add(out$tpar_comp) <- "  // merge thresholds\n"
     grj <- seq_along(groups)
     grj <- glue("Kthres_start{resp}[{grj}]:Kthres_end{resp}[{grj}]")
-    str_add(out$model_comp_basic) <- cglue(
+    str_add(out$tpar_comp) <- cglue(
       "  merged_Intercept{p}{stz}[{grj}] = Intercept{p}{stz}{gr};\n"
     )
+    str_add(out$pll_args) <- cglue(", vector merged_Intercept{p}{stz}")
+  } else {
+    str_add(out$pll_args) <- glue(", vector Intercept{p}{stz}")
   }
   sub_X_means <- ""
   if (stan_center_X(bterms) && length(all_terms(bterms$fe))) {
@@ -312,7 +345,7 @@ stan_thres <- function(bterms, data, prior, ...) {
 }
 
 # Stan code for the baseline functions of the Cox model
-stan_bhaz <- function(bterms, prior, ...) {
+stan_bhaz <- function(bterms, prior, threads, ...) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   out <- list()
   if (!is_cox(bterms$family)) {
@@ -321,6 +354,7 @@ stan_bhaz <- function(bterms, prior, ...) {
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  slice <- stan_slice(threads)
   str_add(out$data) <- glue(
     "  // data for flexible baseline functions\n",
     "  int Kbhaz{resp};  // number of basis functions\n",
@@ -339,21 +373,25 @@ stan_bhaz <- function(bterms, prior, ...) {
   )
   str_add(out$model_def) <- glue(
     "  // compute values of baseline function\n",
-    "  vector[N{resp}] bhaz{resp} = Zbhaz{resp} * sbhaz{resp};\n",
+    "  vector[N{resp}] bhaz{resp} = Zbhaz{resp}{slice} * sbhaz{resp};\n",
     "  // compute values of cumulative baseline function\n",
-    "  vector[N{resp}] cbhaz{resp} = Zcbhaz{resp} * sbhaz{resp};\n"
+    "  vector[N{resp}] cbhaz{resp} = Zcbhaz{resp}{slice} * sbhaz{resp};\n"
+  )
+  str_add(out$pll_args) <- glue(
+    ", matrix Zbhaz{resp}, matrix Zcbhaz{resp}, vector sbhaz{resp}"
   )
   out
 }
 
 # Stan code specific to mixture families
-stan_mixture <- function(bterms, data, prior) {
+stan_mixture <- function(bterms, data, prior, threads, ...) {
   out <- list()
   if (!is.mixfamily(bterms$family)) {
     return(out)
   }
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
+  n <- stan_nn(threads)
   nmix <- length(bterms$family$mix)
   theta_pred <- grepl("^theta", names(bterms$dpars))
   theta_pred <- bterms$dpars[theta_pred]
@@ -368,17 +406,18 @@ stan_mixture <- function(bterms, data, prior) {
     }
     missing_id <- setdiff(1:nmix, dpar_id(names(theta_pred)))
     str_add(out$model_def) <- glue(
-      "  vector[N{p}] theta{missing_id}{p} = rep_vector(0, N{p});\n",                   
+      "  vector[N{p}] theta{missing_id}{p} = rep_vector(0.0, N{p});\n",                   
       "  real log_sum_exp_theta;\n"      
     )
-    sum_exp_theta <- glue("exp(theta{1:nmix}{p}[n])", collapse = " + ")
+    sum_exp_theta <- glue("exp(theta{1:nmix}{p}{n})", collapse = " + ")
     str_add(out$model_comp_mix) <- glue(
       "  for (n in 1:N{p}) {{\n",
+      stan_nn_def(threads),
       "    // scale theta to become a probability vector\n",
       "    log_sum_exp_theta = log({sum_exp_theta});\n"
     )
     str_add(out$model_comp_mix) <- cglue(
-      "    theta{1:nmix}{p}[n] = theta{1:nmix}{p}[n] - log_sum_exp_theta;\n"
+      "    theta{1:nmix}{p}{n} = theta{1:nmix}{p}{n} - log_sum_exp_theta;\n"
     )
     str_add(out$model_comp_mix) <- "  }\n"
   } else if (length(theta_fix)) {
@@ -390,6 +429,7 @@ stan_mixture <- function(bterms, data, prior) {
     str_add(out$data) <- cglue(
       "  real<lower=0,upper=1> theta{1:nmix}{p};\n"
     )
+    str_add(out$pll_args) <- cglue(", real theta{1:nmix}{p}")
   } else {
     # estimate mixture proportions
     str_add(out$data) <- glue(
@@ -409,6 +449,7 @@ stan_mixture <- function(bterms, data, prior) {
     str_add(out$tpar_comp) <- cglue(
       "  theta{1:nmix}{p} = theta{p}[{1:nmix}];\n"
     )
+    str_add(out$pll_args) <- cglue(", real theta{1:nmix}{p}")
   }
   if (order_intercepts(bterms)) {
     # identify mixtures by ordering the intercepts of their components
