@@ -1,5 +1,8 @@
 context("Tests for brmsfit methods")
 
+# to reduce testing time on CRAN substantially
+skip_on_cran()
+
 expect_range <- function(object, lower = -Inf, upper = Inf, ...) {
   testthat::expect_true(all(object >= lower & object <= upper), ...)
 }
@@ -114,6 +117,10 @@ test_that("conditional_effects has reasonable ouputs", {
     "Cannot use 'spaghetti' and 'surface' at the same time"
   )
   
+  me <- conditional_effects(fit1, effects = "Age:visit", re_formula = NULL)
+  exp_nrow <- 100 * length(unique(fit1$data$visit)) 
+  expect_equal(nrow(me[[1]]), exp_nrow)
+  
   mdata = data.frame(
     Age = c(-0.3, 0, 0.3), 
     count = c(10, 20, 30), 
@@ -789,7 +796,7 @@ test_that("summary has reasonable outputs", {
   expect_output(print(summary1), "Population-Level Effects:")
   expect_output(print(summary1), "Priors:")
   
-  summary5 <- SW(summary(fit5))
+  summary5 <- SW(summary(fit5, robust = TRUE))
   expect_output(print(summary5), "sigma1")
   expect_output(print(summary5), "theta1")
   
@@ -811,7 +818,8 @@ test_that("update has reasonable outputs", {
     Trt = rep(0:1, 9), count = rep(c(5, 17, 28), 6),
     patient = 1, Exp = 4
   )
-  up <- update(fit1, newdata = new_data, save_ranef = FALSE, testmode = TRUE)
+  up <- update(fit1, newdata = new_data, save_pars = save_pars(group = FALSE), 
+               testmode = TRUE)
   expect_true(is(up, "brmsfit"))
   expect_equal(attr(up$data, "data_name"), "new_data")
   # expect_equal(attr(up$ranef, "levels")$visit, c("2", "3", "4"))
@@ -830,10 +838,10 @@ test_that("update has reasonable outputs", {
   expect_error(update(fit1, formula. = ~ . + wrong_var),
                "New variables found: 'wrong_var'")
   
-  up <- update(fit1, save_ranef = FALSE, testmode = TRUE)
+  up <- update(fit1, save_pars = save_pars(group = FALSE), testmode = TRUE)
   expect_true(is(up, "brmsfit"))
   # expect_true("r_1_1" %in% up$exclude)
-  up <- update(fit3, save_mevars = FALSE, testmode = TRUE)
+  up <- update(fit3, save_pars = save_pars(latent = FALSE), testmode = TRUE)
   expect_true(is(up, "brmsfit"))
   # expect_true("Xme_1" %in% up$exclude)
   
