@@ -1,5 +1,7 @@
 context("Tests for emmeans support")
 
+skip_on_cran()
+
 require(emmeans)
 
 SW <- suppressWarnings
@@ -20,3 +22,11 @@ test_that("emmeans returns expected output structure", {
   em <- summary(emmeans(fit4, "x1"))
   expect_equal(nrow(em), 1)
 })
+
+test_that("emmeans supports 'mean' predictions", {
+  em <- summary(emmeans(fit2, "Age", dpar = "mean"))
+  expect_equal(nrow(em), 1)
+  
+  em <- summary(emmeans(fit2, "Age", by = "Trt", dpar = "mean"))
+  expect_equal(nrow(em), 2)
+})
