@@ -6,7 +6,8 @@ SW <- brms:::SW
 
 # parsing the Stan code ensures syntactial correctness of models
 # setting this option to FALSE speeds up testing
-options(parse_stancode = TRUE)
+not_cran <- identical(Sys.getenv("NOT_CRAN"), "true")
+options(brms.parse_stancode = not_cran, brms.backend = "rstan")
 
 test_that("specified priors appear in the Stan code", {
   dat <- data.frame(y = 1:10, x1 = rnorm(10), x2 = rnorm(10), 
@@ -346,15 +347,18 @@ test_that("customized covariances appear in the Stan code", {
   
   scode <- make_stancode(rating ~ treat + (1 + treat | gr(subject, cov = M)), 
                          data = inhaler)
-  expect_match2(scode,
-    "kronecker(Lcov_1, diag_pre_multiply(sd_1, L_1)) * to_vector(z_1)"
-  )
+  expect_match2(scode, "r_1 = scale_r_cor_cov(z_1, sd_1, L_1, Lcov_1);")
   expect_match2(scode, "cor_1[choose(k - 1, 2) + j] = Cor_1[j, k];")
   
-  scode <- make_stancode(rating ~ (1 + treat| gr(subject, cor = FALSE, cov = M)), 
+  scode <- make_stancode(rating ~ (1 + treat | gr(subject, cor = FALSE, cov = M)), 
                          data = inhaler)
-  expect_match2(scode, " r_1_1 = (sd_1[1] * (Lcov_1 * z_1[1]));")
-  expect_match2(scode, " r_1_2 = (sd_1[2] * (Lcov_1 * z_1[2]));")
+  expect_match2(scode, "r_1_1 = (sd_1[1] * (Lcov_1 * z_1[1]));")
+  expect_match2(scode, "r_1_2 = (sd_1[2] * (Lcov_1 * z_1[2]));")
+  
+  inhaler$by <- inhaler$subject %% 2
+  scode <- make_stancode(rating ~ (1 + treat | gr(subject, by = by, cov = M)), 
+                         data = inhaler)
+  expect_match2(scode, "r_1 = scale_r_cor_by_cov(z_1, sd_1, L_1, Jby_1, Lcov_1);")
   
   expect_warning(
     scode <- make_stancode(rating ~ treat + period + carry + (1|subject), 
@@ -387,7 +391,7 @@ test_that("truncation appears in the Stan code", {
 test_that("make_stancode handles models without fixed effects", {
   expect_match2(make_stancode(count ~ 0 + (1|patient) + (1+Trt|visit), 
                              data = epilepsy, family = "poisson"), 
-               "mu = rep_vector(0, N);")
+               "mu = rep_vector(0.0, N);")
 })
 
 test_that("make_stancode correctly restricts FE parameters", {
@@ -494,13 +498,6 @@ test_that("self-defined functions appear in the Stan code", {
     make_stancode(time ~ age + arma(cov = TRUE), data = kidney),
     "matrix cholesky_cor_arma1(real ar, real ma"
   )
-  
-  # kronecker products
-  expect_match(
-    make_stancode(rating ~ treat + (treat | gr(subject, cov = M)), 
-                  data = inhaler), 
-    "matrix as_matrix.*matrix kronecker"
-  )
 })
 
 test_that("invalid combinations of modeling options are detected", {
@@ -598,18 +595,18 @@ test_that("Stan code for categorical models is correct", {
   scode <- make_stancode(y ~ x + (1 | gr(.g, id = "ID")), data = dat, 
                          family = categorical(), prior = prior)
   expect_match2(scode, "target += categorical_logit_lpmf(Y[n] | mu[n]);")
-  expect_match2(scode, "mu[n] = [0, mu2[n], mu3[n], muab[n]]';")
+  expect_match2(scode, "mu[n] = transpose([0, mu2[n], mu3[n], muab[n]]);")
   expect_match2(scode, "mu2 = Intercept_mu2 + Xc_mu2 * b_mu2;")
   expect_match2(scode, "muab[n] += r_1_muab_3[J_1[n]] * Z_1_muab_3[n];")
   expect_match2(scode, "target += normal_lpdf(b_mu2 | 0, 10);")
   expect_match2(scode, "target += normal_lpdf(b_muab | 0, 5);")
   expect_match2(scode, "target += cauchy_lpdf(Intercept_mu2 | 0, 1);")
   expect_match2(scode, "target += normal_lpdf(Intercept_mu3 | 0, 2);")
-  expect_match2(scode, "r_1 = (diag_pre_multiply(sd_1, L_1) * z_1)';")
+  expect_match2(scode, "r_1 = scale_r_cor(z_1, sd_1, L_1);")
   
   scode <- make_stancode(y ~ x + (1 |ID| .g), data = dat, 
                          family = categorical(refcat = NA))
-  expect_match2(scode, "mu[n] = [mu1[n], mu2[n], mu3[n], muab[n]]';")
+  expect_match2(scode, "mu[n] = transpose([mu1[n], mu2[n], mu3[n], muab[n]]);")
 })
 
 test_that("Stan code for multinomial models is correct", {
@@ -755,7 +752,7 @@ test_that("Stan code of ordinal models is correct", {
   expect_match2(scode, "matrix[Kcs, nthres] bcs;")
   expect_match2(scode, "mucs = Xcs * bcs;")
   expect_match2(scode, 
-    "target += sratio_logit_lpmf(Y[n] | mu[n], disc, Intercept - mucs[n]');"
+    "target += sratio_logit_lpmf(Y[n] | mu[n], disc, Intercept - transpose(mucs[n]));"
   )
   
   scode <- make_stancode(y ~ x1 + cse(x2) + (cse(1)|g), dat, family = acat())
@@ -768,7 +765,7 @@ test_that("Stan code of ordinal models is correct", {
     paste("mucs[n, 3] = mucs[n, 3] + r_1_3[J_1[n]] * Z_1_3[n]", 
           "+ r_1_6[J_1[n]] * Z_1_6[n];"))
   expect_match2(scode, 
-    "target += acat_probit_lpmf(Y[n] | mu[n], disc, Intercept - mucs[n]');"
+    "target += acat_probit_lpmf(Y[n] | mu[n], disc, Intercept - transpose(mucs[n]));"
   )
   
   # sum-to-zero thresholds
@@ -949,7 +946,7 @@ test_that("Stan code for non-linear models is correct", {
   scode <- make_stancode(bf(y ~ a - exp(b + z), flist = flist, 
                             nl = TRUE, loop = FALSE), 
                          data = data, prior = prior)
-  expect_match2(scode, "\n  mu = nlp_a - exp(nlp_b + C_1);")
+  expect_match2(scode, "mu = nlp_a - exp(nlp_b + C_1);")
   
   flist <- list(a1 ~ 1, a2 ~ z + (x|g))
   prior <- c(set_prior("beta(1,1)", nlpar = "a1", lb = 0, ub = 1),
@@ -1188,10 +1185,12 @@ test_that("weighted, censored, and truncated likelihoods are correct", {
     "target += weights[n] * (binomial_logit_lpmf(Y[n] | trials[n], mu[n]));"
   )
   
-  expect_match2(make_stancode(y | cens(x, y2) ~ 1, dat, poisson()),
-                "target += poisson_lpmf(Y[n] | mu[n]);")
-  expect_match2(make_stancode(y | cens(x) ~ 1, dat, weibull()), 
-                "target += weibull_lccdf(Y[n] | shape, mu[n]);")
+  scode <- make_stancode(y | cens(x, y2) ~ 1, dat, poisson())
+  expect_match2(scode, "target += poisson_lpmf(Y[n] | mu[n]);")
+  
+  scode <- make_stancode(y | cens(x) ~ 1, dat, weibull())
+  expect_match2(scode, "target += weibull_lccdf(Y[n] | shape, mu[n]);")
+  
   dat$x[1] <- 2
   scode <- make_stancode(y | cens(x, y2) ~ 1, dat, gaussian())
   expect_match2(scode, paste0(
@@ -1241,7 +1240,7 @@ test_that("noise-free terms appear in the Stan code", {
   expect_match2(scode, "target += normal_lpdf(meanme_1 | 0, 10)")
   expect_match2(scode, "target += cauchy_lpdf(sdme_1[2] | 0, 5)")
   expect_match2(scode, "target += lkj_corr_cholesky_lpdf(Lme_1 | 2)")
-  expect_match2(scode, "+ (diag_pre_multiply(sdme_1, Lme_1) * zme_1)'")
+  expect_match2(scode, "+ transpose(diag_pre_multiply(sdme_1, Lme_1) * zme_1)")
   expect_match2(scode, "corme_1[choose(k - 1, 2) + j] = Corme_1[j, k];")
   
   scode <- make_stancode(
@@ -1314,7 +1313,7 @@ test_that("by variables in grouping terms are handled correctly", {
     z = factor(rep(c(0, 4.5, 3, 2, 5), each = 20))
   )
   scode <- make_stancode(y ~ x + (1 | gr(g, by = z)), dat)
-  expect_match2(scode, "r_1_1 = (sd_1[1, Jby_1]' .* (z_1[1]));")
+  expect_match2(scode, "r_1_1 = (transpose(sd_1[1, Jby_1]) .* (z_1[1]));")
   scode <- make_stancode(y ~ x + (x | gr(g, by = z)), dat)
   expect_match2(scode, "r_1 = scale_r_cor_by(z_1, sd_1, L_1, Jby_1);")
   expect_match2(scode, "target += student_t_lpdf(to_vector(sd_1) | 3, 0, 2.5);")
@@ -1327,7 +1326,7 @@ test_that("Group syntax | and || is handled correctly,", {
   scode <- make_stancode(y ~ x + (1+x||g1) + (I(x/4)|g2), data)
   expect_match2(scode, "r_1_2 = (sd_1[2] * (z_1[2]));")
   expect_match2(scode, "r_2_1 = r_2[, 1];")
-  expect_match2(scode, "r_2 = (diag_pre_multiply(sd_2, L_2) * z_2)';")
+  expect_match2(scode, "r_2 = scale_r_cor(z_2, sd_2, L_2);")
 })
 
 test_that("predicting zi and hu works correctly", {
@@ -1469,10 +1468,10 @@ test_that("Stan code of Cox models is correct", {
 test_that("offsets appear in the Stan code", {
   data <- data.frame(y = rnorm(10), x = rnorm(10), c = 1)
   scode <- make_stancode(y ~ x + offset(c), data)
-  expect_match2(scode, "Intercept + Xc * b + offsets;")
+  expect_match2(scode, "+ offsets;")
   scode <- make_stancode(bf(y ~ a, a ~ offset(log(c + 1)), nl = TRUE),
                          data, prior = prior(normal(0,1), nlpar = a))
-  expect_match2(scode, "X_a * b_a + offsets_a;")
+  expect_match2(scode, "+ offsets_a;")
 })
 
 test_that("prior only models are correctly checked", {
@@ -1528,7 +1527,7 @@ test_that("Stan code of mixture model is correct", {
   scode <- make_stancode(bf(y ~ x, theta1 ~ x, theta3 ~ x), 
                          data = data, family = fam)
   expect_match2(scode, "log_sum_exp_theta = log(exp(theta1[n]) + exp(theta2[n]) + exp(theta3[n]));")
-  expect_match2(scode, "theta2 = rep_vector(0, N);")
+  expect_match2(scode, "theta2 = rep_vector(0.0, N);")
   expect_match2(scode, "theta3[n] = theta3[n] - log_sum_exp_theta;")
   expect_match2(scode, "ps[1] = theta1[n] + normal_lpdf(Y[n] | mu1[n], sigma1);")
   
@@ -1630,19 +1629,22 @@ test_that("Stan code for Gaussian processes is correct", {
                     z = factor(rep(3:6, each = 10)))
   
   prior <- prior(gamma(0.1, 0.1), sdgp)
-  scode <- make_stancode(y ~ gp(x1) + gp(x2, by = x1), dat, prior = prior)
+  scode <- make_stancode(y ~ gp(x1) + gp(x2, by = x1, gr = FALSE), 
+                         dat, prior = prior)
   expect_match2(scode, "target += inv_gamma_lpdf(lscale_1[1]")
   expect_match2(scode, "target += gamma_lpdf(sdgp_1 | 0.1, 0.1)")
-  expect_match2(scode, "Cgp_2 .* gp(Xgp_2, sdgp_2[1], lscale_2[1], zgp_2)")
+  expect_match2(scode, "gp_pred_2 = gp(Xgp_2, sdgp_2[1], lscale_2[1], zgp_2);")
+  expect_match2(scode, "Cgp_2 .* gp_pred_2;")
   
   prior <- prior + prior(normal(0, 1), lscale, coef = gpx1)
   scode <- make_stancode(y ~ gp(x1) + gp(x2, by = x1, gr = TRUE), 
                          data = dat, prior = prior)
   expect_match2(scode, "target += normal_lpdf(lscale_1[1][1] | 0, 1)")
-  expect_match2(scode, "+ Cgp_2 .* gp(Xgp_2, sdgp_2[1], lscale_2[1], zgp_2)[Jgp_2]")
+  expect_match2(scode, "gp_pred_2 = gp(Xgp_2, sdgp_2[1], lscale_2[1], zgp_2);")
+  expect_match2(scode, "+ Cgp_2 .* gp_pred_2[Jgp_2]")
   
   # non-isotropic GP
-  scode <- make_stancode(y ~ gp(x1, x2, by = z, iso = FALSE), dat)
+  scode <- make_stancode(y ~ gp(x1, x2, by = z, iso = FALSE), data = dat)
   expect_match2(scode, "target += inv_gamma_lpdf(lscale_1[1][2]")
   expect_match2(scode, "target += inv_gamma_lpdf(lscale_1[4][2]")
   
@@ -1650,22 +1652,18 @@ test_that("Stan code for Gaussian processes is correct", {
   scode <- make_stancode(y ~ gp(x1, x2) + gp(x1, by = z, gr = FALSE), 
                          dat, silent = TRUE)
   expect_match2(scode, "gp(Xgp_1, sdgp_1[1], lscale_1[1], zgp_1)")
-  expect_match2(scode, paste0(
-    "mu[Igp_2_2] = mu[Igp_2_2] + Cgp_2_2 .* gp(Xgp_2_2, ", 
-    "sdgp_2[2], lscale_2[2], zgp_2_2);"
-  ))
+  expect_match2(scode, "mu[Igp_2_2] += Cgp_2_2 .* gp_pred_2_2;")
   
   # approximate GPS
   scode <- make_stancode(
-    y ~ gp(x1, k = 10, c = 5/4) + gp(x2, by = x1, k = 10, c = 5/4), dat
+    y ~ gp(x1, k = 10, c = 5/4) + gp(x2, by = x1, k = 10, c = 5/4), 
+    data = dat
   )
   expect_match2(scode, "target += inv_gamma_lpdf(lscale_1")
   expect_match2(scode, 
-    "gpa(Xgp_1, sdgp_1[1], lscale_1[1], zgp_1, slambda_1)"           
-  )
-  expect_match2(scode, 
-    "Cgp_2 .* gpa(Xgp_2, sdgp_2[1], lscale_2[1], zgp_2, slambda_2)"
+    "rgp_1 = sqrt(spd_cov_exp_quad(slambda_1, sdgp_1[1], lscale_1[1])) .* zgp_1;"
   )
+  expect_match2(scode, "Cgp_2 .* gp_pred_2[Jgp_2]")
   
   prior <- c(prior(normal(0, 10), lscale, coef = gpx1, nlpar = a),
              prior(gamma(0.1, 0.1), sdgp, nlpar = a),
@@ -1680,11 +1678,9 @@ test_that("Stan code for Gaussian processes is correct", {
   scode <- make_stancode(bf(y ~ a, a ~ gp(x1, by = z, gr = TRUE), nl = TRUE),
                          data = dat, prior = prior, silent = TRUE)
   expect_match2(scode, 
-    "nlp_a[Igp_a_1_1] = nlp_a[Igp_a_1_1] + Cgp_a_1_1 .* gp(Xgp_a_1_1,"
-  )
-  expect_match2(scode,
-    "gp(Xgp_a_1_3, sdgp_a_1[3], lscale_a_1[3], zgp_a_1_3)[Jgp_a_1_3]"             
+    "nlp_a[Igp_a_1_1] += Cgp_a_1_1 .* gp_pred_a_1_1[Jgp_a_1_1];"
   )
+  expect_match2(scode, "gp(Xgp_a_1_3, sdgp_a_1[3], lscale_a_1[3], zgp_a_1_3)")
   expect_match2(scode, "target += gamma_lpdf(lscale_a_1[3][1] | 2, 2);")
   expect_match2(scode, "target += std_normal_lpdf(zgp_a_1_3);")
   
@@ -1785,21 +1781,21 @@ test_that("Stan code for skew_normal models is correct", {
   dat = data.frame(y = rnorm(10), x = rnorm(10))
   scode <- make_stancode(y ~ x, dat, skew_normal())
   expect_match2(scode, "delta = alpha / sqrt(1 + alpha^2);")
-  expect_match2(scode, "omega = sigma / sqrt(1 - sqrt_2_div_pi^2 * delta^2);")
-  expect_match2(scode, "mu[n] = mu[n] - omega * delta * sqrt_2_div_pi;")
+  expect_match2(scode, "omega = sigma / sqrt(1 - sqrt(2 / pi())^2 * delta^2);")
+  expect_match2(scode, "mu[n] = mu[n] - omega * delta * sqrt(2 / pi());")
   
   scode <- make_stancode(bf(y ~ x, sigma ~ x), dat, skew_normal())
-  expect_match2(scode, "omega[n] = sigma[n] / sqrt(1 - sqrt_2_div_pi^2 * delta^2);")
-  expect_match2(scode, "mu[n] = mu[n] - omega[n] * delta * sqrt_2_div_pi;")
+  expect_match2(scode, "omega[n] = sigma[n] / sqrt(1 - sqrt(2 / pi())^2 * delta^2);")
+  expect_match2(scode, "mu[n] = mu[n] - omega[n] * delta * sqrt(2 / pi());")
   
   scode <- make_stancode(bf(y | se(x) ~ x, alpha ~ x), dat, skew_normal())
   expect_match2(scode, "delta[n] = alpha[n] / sqrt(1 + alpha[n]^2);")
-  expect_match2(scode, "omega[n] = se[n] / sqrt(1 - sqrt_2_div_pi^2 * delta[n]^2);")
-  expect_match2(scode, "mu[n] = mu[n] - omega[n] * delta[n] * sqrt_2_div_pi;")
+  expect_match2(scode, "omega[n] = se[n] / sqrt(1 - sqrt(2 / pi())^2 * delta[n]^2);")
+  expect_match2(scode, "mu[n] = mu[n] - omega[n] * delta[n] * sqrt(2 / pi());")
   
   scode <- make_stancode(y ~ x, dat, mixture(skew_normal, nmix = 2))
-  expect_match2(scode, "omega1 = sigma1 / sqrt(1 - sqrt_2_div_pi^2 * delta1^2);")
-  expect_match2(scode, "mu2[n] = mu2[n] - omega2 * delta2 * sqrt_2_div_pi;")
+  expect_match2(scode, "omega1 = sigma1 / sqrt(1 - sqrt(2 / pi())^2 * delta1^2);")
+  expect_match2(scode, "mu2[n] = mu2[n] - omega2 * delta2 * sqrt(2 / pi());")
 })
 
 test_that("Stan code for missing value terms works correctly", {
@@ -1831,7 +1827,7 @@ test_that("Stan code for missing value terms works correctly", {
   bform <- bf(y ~ 1, sigma ~ 1) + bf(x | mi() ~ 1) + set_rescor(TRUE)
   scode <- make_stancode(bform, dat)
   expect_match2(scode, "Yl[n][2] = Yl_x[n];")
-  expect_match2(scode, "sigma[n] = [sigma_y[n], sigma_x]';")
+  expect_match2(scode, "sigma[n] = transpose([sigma_y[n], sigma_x]);")
   expect_match2(scode, "LSigma[n] = diag_pre_multiply(sigma[n], Lrescor);")
   
   bform <- bf(x | mi() ~ y, family = "lognormal")
@@ -2052,7 +2048,7 @@ test_that("student-t group-level effects work without errors", {
     epilepsy, prior = bprior
   )
   expect_match2(scode,
-    "rep_matrix(dfm_1, M_1) .* (diag_pre_multiply(sd_1, L_1) * z_1)';"
+    "r_1 = rep_matrix(dfm_1, M_1) .* scale_r_cor(z_1, sd_1, L_1);"
   )
   expect_match2(scode, "target += normal_lpdf(df_1 | 20, 5);")
 })
@@ -2095,3 +2091,72 @@ test_that("to_vector() is correctly removed from prior of SD parameters", {
   expect_match2(scode, "prior_sd_1_1 = normal_rng(0,0.1);")
   expect_match2(scode, "prior_sd_1_2 = normal_rng(0,0.01);")
 })
+
+test_that("threaded Stan code is correct", {
+  dat <- data.frame(
+    count = rpois(236, lambda = 20),
+    visit = rep(1:4, each = 59),
+    patient = factor(rep(1:59, 4)),
+    Age = rnorm(236), 
+    Trt = factor(sample(0:1, 236, TRUE)),
+    AgeSD = abs(rnorm(236, 1)),
+    Exp = sample(1:5, 236, TRUE),
+    volume = rnorm(236),
+    gender = factor(c(rep("m", 30), rep("f", 29)))
+  )
+  
+  # only parse models if cmdstan can be found on the system
+  found_cmdstan <- !is(try(cmdstan_version(), silent = TRUE), "try-error")
+  options(brms.parse_stancode = found_cmdstan && not_cran, 
+          brms.backend = "cmdstanr")
+  threads <- threading(2, grainsize = 20)
+  
+  bform <- bf(
+    count ~ Trt*Age + mo(Exp) + s(Age) + offset(Age) + (1+Trt|visit),
+    sigma ~ Trt + gp(Age)
+  )
+  scode <- make_stancode(bform, dat, family = student(), threads = threads)
+  expect_match2(scode, "real partial_log_lik(int[] seq, int start,")
+  expect_match2(scode, "mu[n] += (bsp[1]) * mo(simo_1, Xmo_1[nn])")
+  expect_match2(scode, "ptarget += student_t_lpdf(Y[start:end] | nu, mu, sigma);")
+  expect_match2(scode, "+ gp_pred_sigma_1[Jgp_sigma_1[start:end]]")
+  expect_match2(scode, "target += reduce_sum(partial_log_lik, seq, grainsize, Y,")
+  
+  scode <- make_stancode(
+    visit ~ cs(Trt) + Age, dat, family = sratio(), 
+    threads = threads,
+  )
+  expect_match2(scode, "matrix[N, nthres] mucs = Xcs[start:end] * bcs;")
+  expect_match2(scode, 
+    "ptarget += sratio_logit_lpmf(Y[nn] | mu[n], disc, Intercept - transpose(mucs[n]));"
+  )
+  
+  scode <- make_stancode(
+    bf(visit ~ a * Trt ^ b, a ~ mo(Exp), b ~ s(Age), nl = TRUE),
+    data = dat, family = Gamma("log"), 
+    prior = set_prior("normal(0, 1)", nlpar = c("a", "b")),
+    threads = threads
+  )
+  expect_match2(scode, "mu[n] = shape * exp(-(nlp_a[nn] * C_1[nn] ^ nlp_b[nn]));")
+  expect_match2(scode, "ptarget += gamma_lpdf(Y[start:end] | shape, mu);")
+  
+  bform <- bf(mvbind(count, Exp) ~ Trt) + set_rescor(TRUE)
+  scode <- make_stancode(bform, dat, gaussian(), threads = threads)
+  expect_match2(scode, "ptarget += multi_normal_cholesky_lpdf(Y[start:end] | Mu, LSigma);")
+  
+  bform <- bf(mvbind(count, Exp) ~ Trt) + set_rescor(FALSE)
+  scode <- make_stancode(bform, dat, gaussian(), threads = threads)
+  expect_match2(scode, "target += reduce_sum(partial_log_lik_count, seq_count,")
+  expect_match2(scode, "target += reduce_sum(partial_log_lik_Exp, seq_Exp,")
+  expect_match2(scode, 
+    "ptarget += normal_id_glm_lpdf(Y_Exp[start:end] | Xc_Exp[start:end], Intercept_Exp, b_Exp, sigma_Exp);"
+  )
+  
+  scode <- make_stancode(
+    visit ~ Trt, dat, family = mixture(poisson(), nmix = 2),
+    threads = threading(4, grainsize = 10, static = TRUE)
+  )
+  expect_match2(scode, "ps[1] = log(theta1) + poisson_log_lpmf(Y[nn] | mu1[n]);")
+  expect_match2(scode, "ptarget += log_sum_exp(ps);")
+  expect_match2(scode, "target += reduce_sum_static(partial_log_lik,")
+})
